# Makefile.inc
# 
# This include file defines the Fortran compilers available to the
# developers and sets up the relevant compiler flags and patterns.
# Additional commands are also included although these may not be
# necessary for all projects.
# 
# We illustrate how make (actually gmake) may be used to allow a
# default compiler and associated flags either of which may be
# overridden by providing flags to the make command.
# 
# This file should only need to be set up once for a particular
# development environment. As an illustration we illustrate
# how more than one compiler can be invoked by the use of
# additional flags to the invoking make command.
# 
# We assume that the same Fortran compiler can be used to compile
# both fixed format and free format Fortran source code based on
# the source code file suffix. (Here we have assumed .f for fixed
# and .f95 or .f90 for free format.) For flexibility we allow for
# different sets of compiler flags for the two types of source
# files.
# 
# We also assume that the system compiles individual source files
# into object files with the same filestem and a .o suffix. These
# in turn are linked together to form the executable.
#
# We also specify the lapack and blas libraries when linking. 
# This involves hard-wired paths and could possibly be improved
# slightly by using variables to specify paths. When editing this
# file you will need to look for the string "LIBS" and change
# any occurrences of path information to ones relevant to
# your particular set-up.
# 
# Switch off all suffix rules. This is probably the safest course
# of action as some of the built-in rules can interfere when using
# certain compilers. For example, some new Fortran compilers
# produce a .mod file (as well as a .o file) when compiling source
# files containing modules. On some Unix systems make has a
# built-in rule for compiling .mod files as Modula 2 source files
# (never heard of Modula 2? Don't worry!).
# 
.SUFFIXES:

# 
#  If you just want to switch off the default rule for Modula 2
#  you could use
# 
%.o : %.mod

# 
# Set up the pattern rules required for compiling free and fixed
# form Fortran source files along with the compilation of C source
# files (required only if interoperability with C is required).
# Note that interoperability with C requires a compatible C
# compiler -- see your Fortran compiler documentation for
# guidance.
# 
# Generate .o files from free format source files (assume  
# either a .f95 or .f90 suffix).
#
%.o: %.f95
	$(F95) $(F95FLAGS) -o $@ -c $<

%.o: %.f90
	$(F95) $(F95FLAGS) -o $@ -c $<

#
# and for fixed format source (assume a .f suffix)
#
%.o: %.f
	$(F95) $(F77FLAGS) -o $@ -c $<

#
# assume that .F90 suffix requires preprocessing to
# generate a .f90 source file.
#
%.f90: %.F90
	$(FPP) $(FPPFLAGS) $< $@

#
# and for C -- use a compatible C compiler that will allow
# interoperability with the chosen Fortran compiler. For NAG
# this is gcc (same for gfortran)
#
%.o: %.c
	$(CC) $(CCFLAGS) -o $@ -c $<

#
# Generate an executable file of the same filestem name as the
# source file containing the main program. For example, the
# executable driver would be generated by linking driver.o
# (obtained from driver.f or driver.f95 and containing the main
# program) with the required object files. For more details of
# how this works see various examples in the chapetr makefiles.
# Assume that we link with the free format compiler flags.
#
%: %.o
	$(F95) $(F95LINKFLAGS) -o $@ $^ $(LIBS)

# Now define the compiler and option flags. We use the ability to
# define make variables on the command line to allow the default
# choices to be overridden.
# 
# Here we use the NAG compiler with checking switched on as the
# default and we allow gfortran as an alternative compiler via the
# setting of make variables within the command line. We also allow
# the default checking compilation mode to be overridden to
# provide optimized executables in both cases.
# 
# The make variable COMPILE_OPT controls the compilation mode. If
# this is defined via the command line (or via the use of
# environmental variables) then optimized code is generated
# otherwise the default checking mode is used.
# 
# This allows the following command line combinations
# 
# 1.   make
# 
#   Use the default compiler (NAG) and default options (checking)
# 
# 2.   make COMPILE_OPT=optimized
# 
#   Use the default compiler (NAG) and the compiler flags for
#   producing optimized code.
# 
# 3.   make COMPILER=gfortran
# 
#   Use the default compiler options (set for checking) but
#   override the default compiler to use gfortran.
# 
# 4.   make COMPILER=gfortran COMPILE_OPT=optimized
# 
#   Override both the compiler (use gfortran) and the compiler
#   options (produce optimized code).
        
# Check if COMPILE_OPT has been explicitly defined; if not use
# the default.

ifdef COMPILE_OPT
  COMPILE_OPT = optimized
else
  COMPILE_OPT = checking
endif

# Set up options for code coverage using the gfortran compiler.
# Note that some codes that run under Nag may not run under
# gfortran as gfortran sometimes lags behind in implementing
# new features.

ifdef COVERAGE
  COMPILER = gfortran
  COVERAGE = yes
else
  COVERAGE = no
endif

# Check if COMPILER has been explicitly defined; if not use
# the default.
#
# VALID_COMP is used to ensure that we are given a valid
# compiler name. This allows us to pick up typos in the command
# line and exit gracefully with an informative error message.

ifndef COMPILER
  COMPILER = nagfor
  VALID_COMP = yes
else
  VALID_COMP = no
endif

ifeq "${COMPILER}" "nagfor"
  ifeq "${COMPILE_OPT}" "checking"
#
# Flags to turn on runtime checks, etc for the NAG compiler
#
# -g: Produce  information  for interactive debugging by the host
#     system debugger.
#
# -g90: Produce  information  for dbx, a Fortran aware fromt end to
#       the host system debugger (needs to be specifies when 
#       compiling and linking).
#
# -gline: generate source line information for execution backtrace
#
# -C=all: perform a variety of run-time checks including array bounds
#
# -C=undefined: Check for undefined  variables.
#   NOTE: code compiled with this option set is not binary
#   compatible with code compiled without this option. There are
#   also problems when using IEEE intrinsic modules.
#
# (-C=undefined has to be requested explicitly -- i.e., it
# is not included in "all" as it is considered expensive.
# However this type of error is very difficult to find
# without the run time checks.)
#
# -u: Specify that IMPLICIT NONE is in effect by default,  unless
#     overridden by explicit IMPLICIT statements.
#
# -nan: Initialise REAL and COMPLEX  variables  to  IEEE  Signalling  NaN,
#       causing  a  runtime crash if the values are used before being set.
#       This affects local variables, module variables,  and  INTENT(OUT)
#       dummy  arguments  only; it does not affect variables in COMMON
#       or EQUIVALENCE.
#
    F95FLAGS = -g -gline -g90 -u -nan -C=all -C=undefined -C=dangling
    F77FLAGS = -g -gline -g90 -u -nan -C=all -C=undefined -C=dangling
    F95LINKFLAGS = $(F95FLAGS) 
    F77LINKFLAGS = $(F77FLAGS) 
    FPP = /usr/local/lib/NAG_Fortran/fpp
    FPPFLAGS = -DIEEE -e -P
    CC = gcc
    CCFLAGS =
# Define LIBS for Fortran libraries for Lapack and Blas compiled
# for checking
    LIBS = /home/trh/Libs/NagFullDebug/lapack.a  \
           /home/trh/Libs/NagFullDebug/blas.a
  else
    F95FLAGS = -O3
    F77FLAGS = -O3
    F95LINKFLAGS = $(F95FLAGS) -unsharedrts
    F77LINKFLAGS = $(F77FLAGS) -unsharedrts
    FPP = /usr/local/lib/NAG_Fortran/fpp
    FPPFLAGS = -DIEEE -e -P
    CC = gcc
    CCFLAGS =
# Define LIBS for Fortran libraries for Lapack and Blas compiled
# using the optimized options 
    LIBS = /home/trh/Libs/NagOptF90/lapack.a  \
           /home/trh/Libs/NagOptF90/blas.a
  endif
endif

#
# Do the same for the gfortran compiler
#
# -Wall: Provide warnings of usages that are probably best
#        avoided
#
# -fcheck=all: Enable a set of run-time checks including array
#        subscript checking, checks on pointers, memory
#        allocation checks, etc
#
# -ffpe-trap=invalid,zero,overflow: Trap floating point
#        exceptions generated by invalid operations (e.g., SQRT(-1.0)),
#        division by zero and floating-point overflow.
#
# -fbacktrace: In the case of a serious error attempt to
#        generate a backtrace showing where the error occurred and what
#        routines were involved.
#
# -g: Add information for the gnu debugger (gdb)
#

ifeq "${COMPILER}" "gfortran"
  VALID_COMP = yes
  ifeq "${COMPILE_OPT}" "checking"
    F95FLAGS = -Wall -fcheck=all -g -fbacktrace \
               -ffpe-trap=invalid,zero,overflow
    F77FLAGS = -Wall -fcheck=all -g -fbacktrace \
               -ffpe-trap=invalid,zero,overflow
    F95LINKFLAGS = $(F95FLAGS)
    F77LINKFLAGS = $(F77FLAGS)

    ifeq "$(COVERAGE)" "yes"
      F95FLAGS = -Wall -coverage
      F77FLAGS = -Wall -coverage
    endif

    FPP = cpp
    FPPFLAGS = -P
    CC = gcc
    CCFLAGS =
#
# Define LIBS for Fortran libraries for Lapack and Blas compiled
# using the optimized options and gfortran. gfortran allows
# optimized and checking code to be linked.
#
    LIBS = /home/trh/Libs/GfortranOptF90/lapack.a  \
           /home/trh/Libs/GfortranOptF90/blas.a
  else
    F95FLAGS = -O3
    F77FLAGS = -O3
    F95LINKFLAGS = $(F95FLAGS)
    F77LINKFLAGS = $(F77FLAGS)
    FPP = cpp
    FPPFLAGS = -P
    CC = gcc
    CCFLAGS =
#
# Define LIBS for Fortran libraries for Lapack and Blas compiled
# using the optimized options and gfortran. gfortran allows
# optimized and checking code to be linked. 
#
    LIBS = /home/trh/Libs/GfortranOptF90/lapack.a  \
           /home/trh/Libs/GfortranOptF90/blas.a
  endif
endif

#
# Check that we have found a valid compiler. If not output an
# error message and stop.
#
ifeq "${VALID_COMP}" "no"
  $(error "Illegal compiler name provided: ${COMPILER}.  Aborting.")
endif

F95 = ${COMPILER}

#
# Set the commands for building random archive libraries (this
# is Unix specific)
#
AR = ar
ARFLAGS = rv

#
# Some older systems might require RANLIB to be used here to
# build the indexes required for the archive library. Comment
# out RANLIB assignment below if required (put your local
# command name in for ranlib!)
#
# RANLIB = ranlib

